package java.util.concurrent.atomic;

import sun.misc.Unsafe;
import java.io.Serializable;

public class AtomicBoolean implements Serializable
{
    private static final long serialVersionUID = 4654671469794556979L;
    private static final Unsafe unsafe;
    private static final long valueOffset;
    private volatile int value;
    
    public AtomicBoolean(boolean value) {
        this.value = value;
    }
    
    public AtomicBoolean() {
    }
    
    public final boolean get() {
        return this.value != 0;
    }
    
    public final boolean compareAndSet(boolean expected, boolean x) {
        return AtomicBoolean.unsafe.compareAndSwapInt(this, AtomicBoolean.valueOffset, expected?1:0, x?1:0);
    }
    
    public boolean weakCompareAndSet(boolean expected, boolean x) {
        return AtomicBoolean.unsafe.compareAndSwapInt(this, AtomicBoolean.valueOffset, expected?1:0, x?1:0);
    }
    
    public final void set(boolean value) {
        this.value = value;
    }
    
    public final void lazySet(boolean x) {
        AtomicBoolean.unsafe.putOrderedInt(this, AtomicBoolean.valueOffset, x?1:0);
    }
    
    public final boolean getAndSet(boolean b) {
        int value;
        do {
            value = this.get();
        } while (!this.compareAndSet((boolean)value, b));
        return value;
    }
    
    @Override
    public String toString() {
        return Boolean.toString(this.get());
    }
    
    static {
        unsafe = Unsafe.getUnsafe();
        try {
            valueOffset = AtomicBoolean.unsafe.objectFieldOffset(AtomicBoolean.class.getDeclaredField("value"));
        }
        catch (Exception cause) {
            throw new Error(cause);
        }
    }
}
